@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pkcs11_lib.h 379 2009-12-03 08:40:22Z ludovic.rousseau $
+ * $Id: pkcs11_lib.h 435 2010-06-07 14:53:17Z ludovic.rousseau $
  */
 #ifndef __PKCS11_LIB_H__
 #define __PKCS11_LIB_H__
@@ -29,7 +29,7 @@ typedef struct pkcs11_handle_str pkcs11_handle_t;
 #endif
 
 PKCS11_EXTERN int crypto_init(cert_policy *policy);
-PKCS11_EXTERN int load_pkcs11_module(char *module, pkcs11_handle_t **h);
+PKCS11_EXTERN int load_pkcs11_module(const char *module, pkcs11_handle_t **h);
 PKCS11_EXTERN int init_pkcs11_module(pkcs11_handle_t *h,int flag);
 PKCS11_EXTERN int find_slot_by_number(pkcs11_handle_t *h,unsigned int slot_num,
                                       unsigned int *slot);
@@ -59,6 +59,7 @@ PKCS11_EXTERN int close_pkcs11_session(pkcs11_handle_t *h);
 PKCS11_EXTERN int pkcs11_login(pkcs11_handle_t *h, char *password);
 PKCS11_EXTERN int pkcs11_pass_login(pkcs11_handle_t *h, int nullok);
 PKCS11_EXTERN int get_slot_login_required(pkcs11_handle_t *h);
+PKCS11_EXTERN int get_slot_protected_authentication_path(pkcs11_handle_t *h);
 PKCS11_EXTERN cert_object_t **get_certificate_list(pkcs11_handle_t *h,
                                                   int *ncert);
 PKCS11_EXTERN int get_private_key(pkcs11_handle_t *h, cert_object_t *);
