@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: uri.c 479 2011-01-22 17:37:43Z ludovic.rousseau $
+ * $Id: uri.c 507 2011-11-08 21:35:33Z ludovic.rousseau $
  */
 
 #define __URI_C_
@@ -79,7 +79,7 @@ int is_symlink(const char *path){
 }
 
 
-#ifdef HAVE_CURL
+#ifdef HAVE_CURL_CURL_H
 
 #include <curl/curl.h>
 
