@@ -19,7 +19,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
- * $Id: opensc_mapper.c 396 2010-02-03 17:41:41Z ludovic.rousseau $
+ * $Id: opensc_mapper.c 445 2010-08-14 16:19:36Z ludovic.rousseau $
  */
 
 #define __OPENSC_MAPPER_C_
@@ -135,7 +135,7 @@ static int opensc_mapper_match_user(X509 *x509, const char *user, void *context)
 parses the certificate and return the _first_ user that has it in
 their ${HOME}/.eid/authorized_certificates
 */
-static char * opensc_mapper_find_user(X509 *x509, void *context) {
+static char * opensc_mapper_find_user(X509 *x509, void *context, int *match) {
 	int n = 0;
 	struct passwd *pw = NULL;
 	char *res = NULL;
@@ -155,8 +155,9 @@ static char * opensc_mapper_find_user(X509 *x509, void *context) {
 	    }
 	    /* arriving here means user found */
             DBG1("Certificate match found for user '%s'",pw->pw_name);
-	    res= clone_str(pw->pw_name);
-		    endpwent();
+	    res = clone_str(pw->pw_name);
+	    endpwent();
+	    *match = 1;
 	    return res;
         } /* next login */
 	/* no user found that contains cert in their directory */
