@@ -17,7 +17,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
- * $Id: uid_mapper.c 358 2008-11-06 14:28:46Z ludovic.rousseau $
+ * $Id: uid_mapper.c 445 2010-08-14 16:19:36Z ludovic.rousseau $
  */
 
 #define __UID_MAPPER_C_
@@ -61,7 +61,7 @@ static char ** uid_mapper_find_entries(X509 *x509, void *context) {
 parses the certificate and return the map of the first UID entry found
 If no UID found or map error, return NULL
 */
-static char * uid_mapper_find_user(X509 *x509, void *context) {
+static char * uid_mapper_find_user(X509 *x509, void *context, int *match) {
 	char *res;
 	char **entries= cert_info(x509,CERT_UID,ALGORITHM_NULL);
         if (!entries) {
@@ -69,7 +69,7 @@ static char * uid_mapper_find_user(X509 *x509, void *context) {
             return NULL;
         }
         DBG1("trying to map uid entry '%s'",entries[0]);
-        res = mapfile_find(mapfile,entries[0],ignorecase);
+        res = mapfile_find(mapfile,entries[0],ignorecase,match);
 	if (!res) {
 	    DBG("Error in map process");
 	    return NULL;
