@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pam_pkcs11.c 491 2011-03-22 08:30:44Z ludovic.rousseau $
+ * $Id: pam_pkcs11.c 512 2012-04-07 16:55:20Z ludovic.rousseau $
  */
 
 /* We have to make this definitions before we include the pam header files! */
@@ -72,7 +72,7 @@ static int is_spaced_str(const char *str) {
 /*
  * implement pam utilities for older versions of pam.
  */
-static int pam_prompt(pam_handle_t *pamh, int style, char **response, char *text)
+static int pam_prompt(pam_handle_t *pamh, int style, char **response, char *fmt, ...)
 {
   int rv;
   struct pam_conv *conv;
@@ -81,6 +81,12 @@ static int pam_prompt(pam_handle_t *pamh, int style, char **response, char *text
   /* struct pam_message *(msgp[1]) = { &msg}; */
   struct pam_message *(msgp[1]);
   msgp[0] = &msg;
+  va_list va;
+  char text[128];
+
+  va_start(va, fmt);
+  vsnprintf(text, sizeof text, fmt, va);
+  va_end(va);
 
   msg.msg_style = style;
   msg.msg = text;
@@ -127,7 +133,7 @@ static int pam_get_pwd(pam_handle_t *pamh, char **pwd, char *text, int oitem, in
   struct pam_message msg;
   struct pam_response *resp;
   /* struct pam_message *(msgp[1]) = { &msg}; */
-  struct pam_message *(msgp[1]);
+  const struct pam_message *(msgp[1]);
   msgp[0] = &msg;
 
   /* use stored password if variable oitem is set */
@@ -176,7 +182,6 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
   int i, rv;
   const char *user = NULL;
   char *password;
-  char password_prompt[128];
   unsigned int slot_num = 0;
   int is_a_screen_saver = 0;
   struct configuration_st *configuration;
@@ -194,8 +199,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
   char **issuer, **serial;
   const char *login_token_name = NULL;
 
-  snprintf(password_prompt, sizeof(password_prompt), _("Smartcard authentification starts"));
-  pam_prompt(pamh, PAM_TEXT_INFO , NULL, password_prompt);
+  pam_prompt(pamh, PAM_TEXT_INFO , NULL, _("Smartcard authentification starts"));
 
   /* first of all check whether debugging should be enabled */
   for (i = 0; i < argc; i++)
@@ -270,7 +274,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 	char *service;
 	if (configuration->screen_savers) {
 	    DBG("Is it a screen saver?");
-    	    pam_get_item(pamh, PAM_SERVICE, &service);
+		pam_get_item(pamh, PAM_SERVICE, &service);
 	    for (i=0; configuration->screen_savers[i]; i++) {
 		if (strcmp(configuration->screen_savers[i], service) == 0) {
 		    is_a_screen_saver = 1;
@@ -282,17 +286,16 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 	pkcs11_pam_fail = PAM_CRED_INSUFFICIENT;
 
 	/* look to see if username is already set */
-    	pam_get_item(pamh, PAM_USER, &user);
+	pam_get_item(pamh, PAM_USER, &user);
 	if (user) {
 	    DBG1("explicit username = [%s]", user);
 	}
   } else {
 	rv = pam_get_item(pamh, PAM_USER, &user);
 	if (rv != PAM_SUCCESS || user == NULL || user[0] == '\0') {
-	  snprintf(password_prompt, sizeof(password_prompt),
+	  pam_prompt(pamh, PAM_TEXT_INFO, NULL,
 		  _("Please insert your %s or enter your username."),
 		  _(configuration->token_type));
-	  pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
 	  /* get user name */
 	  rv = pam_get_user(pamh, &user, NULL);
 
@@ -321,8 +324,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     if (!configuration->quiet) {
 		pam_syslog(pamh, LOG_ERR, "load_pkcs11_module() failed loading %s: %s",
 			configuration->pkcs11_modulepath, get_error());
-		snprintf(password_prompt, sizeof(password_prompt), _("Error 2302: PKCS#11 module failed loading"));
-		pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+		pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2302: PKCS#11 module failed loading"));
 		sleep(configuration->err_display_time);
 	}
     return PAM_AUTHINFO_UNAVAIL;
@@ -336,8 +338,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     ERR1("init_pkcs11_module() failed: %s", get_error());
     if (!configuration->quiet) {
 		pam_syslog(pamh, LOG_ERR, "init_pkcs11_module() failed: %s", get_error());
-		snprintf(password_prompt, sizeof(password_prompt), _("Error 2304: PKCS#11 module could not be initialized"));
-		pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+		pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2304: PKCS#11 module could not be initialized"));
 		sleep(configuration->err_display_time);
 	}
     return PAM_AUTHINFO_UNAVAIL;
@@ -356,8 +357,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     ERR("no suitable token available");
     if (!configuration->quiet) {
 		pam_syslog(pamh, LOG_ERR, "no suitable token available");
-		snprintf(password_prompt, sizeof(password_prompt), _("Error 2306: No suitable token available"));
-		pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+		pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2306: No suitable token available"));
 		sleep(configuration->err_display_time);
 	}
 
@@ -370,10 +370,9 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
      * or because we used one to log in */
     if (login_token_name || configuration->wait_for_card) {
       if (login_token_name) {
-        snprintf(password_prompt, sizeof(password_prompt),
+        pam_prompt(pamh, PAM_TEXT_INFO, NULL,
 			_("Please insert your smart card called \"%.32s\"."),
 			login_token_name);
-        pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
       } else {
         pam_prompt(pamh, PAM_TEXT_INFO, NULL,
                  _("Please insert your smart card."));
@@ -392,10 +391,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
         return pkcs11_pam_fail;
       }
     } else if (user) {
-	
 		if (!configuration->quiet) {
-			snprintf(password_prompt, sizeof(password_prompt), _("Error 2308: No smartcard found"));
-			pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+			pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2308: No smartcard found"));
 			sleep(configuration->err_display_time);
 		}
 
@@ -405,10 +402,9 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     } else {
       /* we haven't prompted for the user yet, get the user and see if
        * the smart card has been inserted in the mean time */
-      snprintf(password_prompt, sizeof(password_prompt),
-	  	_("Please insert your %s or enter your username."),
+      pam_prompt(pamh, PAM_TEXT_INFO, NULL,
+	    _("Please insert your %s or enter your username."),
 		_(configuration->token_type));
-      pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
       rv = pam_get_user(pamh, &user, NULL);
 
       /* check one last time for the smart card before bouncing to the next
@@ -423,8 +419,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
       if (rv != 0) {
         /* user gave us a user id and no smart card go to next module */
 		if (!configuration->quiet) {
-			snprintf(password_prompt,  sizeof(password_prompt), _("Error 2310: No smartcard found"));
-			pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+			pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2310: No smartcard found"));
 			sleep(configuration->err_display_time);
 		}
 
@@ -433,17 +428,15 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
       }
     }
   } else {
-      snprintf(password_prompt, sizeof(password_prompt),
+      pam_prompt(pamh, PAM_TEXT_INFO, NULL,
 		  _("%s found."), _(configuration->token_type));
-      pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
   }
   rv = open_pkcs11_session(ph, slot_num);
   if (rv != 0) {
     ERR1("open_pkcs11_session() failed: %s", get_error());
     if (!configuration->quiet) {
 		pam_syslog(pamh, LOG_ERR, "open_pkcs11_session() failed: %s", get_error());
-		snprintf(password_prompt, sizeof(password_prompt), _("Error 2312: open PKCS#11 session failed"));
-		pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+		pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2312: open PKCS#11 session failed"));
 		sleep(configuration->err_display_time);
 	}
     release_pkcs11_module(ph);
@@ -455,22 +448,22 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     ERR1("get_slot_login_required() failed: %s", get_error());
     if (!configuration->quiet) {
 		pam_syslog(pamh, LOG_ERR, "get_slot_login_required() failed: %s", get_error());
-		snprintf(password_prompt, sizeof(password_prompt), _("Error 2314: Slot login failed"));
-		pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+		pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2314: Slot login failed"));
 		sleep(configuration->err_display_time);
 	}
     release_pkcs11_module(ph);
     return pkcs11_pam_fail;
   } else if (rv) {
     /* get password */
-	snprintf(password_prompt, sizeof(password_prompt),
+	pam_prompt(pamh, PAM_TEXT_INFO, NULL,
 		_("Welcome %.32s!"), get_slot_tokenlabel(ph));
-	pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
 
 	/* no CKF_PROTECTED_AUTHENTICATION_PATH */
 	rv = get_slot_protected_authentication_path(ph);
 	if ((-1 == rv) || (0 == rv))
 	{
+		char password_prompt[128];
+
 		snprintf(password_prompt,  sizeof(password_prompt), _("%s PIN: "), _(configuration->token_type));
 		if (configuration->use_first_pass) {
 			rv = pam_get_pwd(pamh, &password, NULL, PAM_AUTHTOK, 0);
@@ -482,8 +475,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 		}
 		if (rv != PAM_SUCCESS) {
 			if (!configuration->quiet) {
-				snprintf(password_prompt, sizeof(password_prompt), _("Error 2316: password could not be read"));
-				pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+				pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2316: password could not be read"));
 				sleep(configuration->err_display_time);
 			}
 			release_pkcs11_module(ph);
@@ -503,8 +495,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 			pam_syslog(pamh, LOG_ERR,
 					"password length is zero but the 'nullok' argument was not defined.");
 			if (!configuration->quiet) {
-				snprintf(password_prompt, sizeof(password_prompt), _("Error 2318: Empty smartcard PIN not allowed."));
-				pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+				pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2318: Empty smartcard PIN not allowed."));
 				sleep(configuration->err_display_time);
 			}
 			return PAM_AUTH_ERR;
@@ -512,9 +503,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 	}
 	else
 	{
-		snprintf(password_prompt, sizeof(password_prompt),
+		pam_prompt(pamh, PAM_TEXT_INFO, NULL,
 			_("Enter your %s PIN on the pinpad"), _(configuration->token_type));
-		pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
 		/* use pin pad */
 		password = NULL;
 	}
@@ -532,9 +522,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     if (rv != 0) {
       ERR1("open_pkcs11_login() failed: %s", get_error());
 		if (!configuration->quiet) {
-        	pam_syslog(pamh, LOG_ERR, "open_pkcs11_login() failed: %s", get_error());
-			snprintf(password_prompt, sizeof(password_prompt), _("Error 2320: Wrong smartcard PIN"));
-			pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+			pam_syslog(pamh, LOG_ERR, "open_pkcs11_login() failed: %s", get_error());
+			pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2320: Wrong smartcard PIN"));
 			sleep(configuration->err_display_time);
 		}
       goto auth_failed_nopw;
@@ -546,8 +535,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     ERR1("get_certificate_list() failed: %s", get_error());
     if (!configuration->quiet) {
 		pam_syslog(pamh, LOG_ERR, "get_certificate_list() failed: %s", get_error());
-		snprintf(password_prompt, sizeof(password_prompt), _("Error 2322: No certificate found"));
-		pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+		pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2322: No certificate found"));
 		sleep(configuration->err_display_time);
 	}
     goto auth_failed_nopw;
@@ -560,10 +548,9 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
   for (i = 0; i < ncert; i++) {
     X509 *x509 = (X509 *)get_X509_certificate(cert_list[i]);
     if (!x509 ) continue; /* sanity check */
-    DBG1("verifing the certificate #%d", i + 1);
+    DBG1("verifying the certificate #%d", i + 1);
 	if (!configuration->quiet) {
-		snprintf(password_prompt, sizeof(password_prompt), _("verifying certificate"));
-		pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
+		pam_prompt(pamh, PAM_TEXT_INFO, NULL, _("verifying certificate"));
 	}
 
       /* verify certificate (date, signature, CRL, ...) */
@@ -575,19 +562,22 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
                    "verify_certificate() failed: %s", get_error());
 			switch (rv) {
 				case -2: // X509_V_ERR_CERT_HAS_EXPIRED:
-					snprintf(password_prompt, sizeof(password_prompt), _("Error 2324: Certificate has expired"));
+					pam_prompt(pamh, PAM_ERROR_MSG , NULL,
+						_("Error 2324: Certificate has expired"));
 					break;
 				case -3: // X509_V_ERR_CERT_NOT_YET_VALID:
-					snprintf(password_prompt, sizeof(password_prompt), _("Error 2326: Certificate not yet valid"));
+					pam_prompt(pamh, PAM_ERROR_MSG , NULL,
+						_("Error 2326: Certificate not yet valid"));
 					break;
 				case -4: // X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
-					snprintf(password_prompt, sizeof(password_prompt), _("Error 2328: Certificate signature invalid"));
+					pam_prompt(pamh, PAM_ERROR_MSG , NULL,
+						_("Error 2328: Certificate signature invalid"));
 					break;
 				default:
-					snprintf(password_prompt, sizeof(password_prompt), _("Error 2330: Certificate invalid"));
+					pam_prompt(pamh, PAM_ERROR_MSG , NULL,
+						_("Error 2330: Certificate invalid"));
 					break;
 			}
-			pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
 			sleep(configuration->err_display_time);
 		}
 	goto auth_failed_nopw;
@@ -614,14 +604,13 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 	} else {
           DBG1("certificate is valid and matches user %s",user);
 	  /* try to set up PAM user entry with evaluated value */
-  	  rv = pam_set_item(pamh, PAM_USER,(const void *)user);
+	  rv = pam_set_item(pamh, PAM_USER,(const void *)user);
 	  if (rv != PAM_SUCCESS) {
 	    ERR1("pam_set_item() failed %s", pam_strerror(pamh, rv));
             if (!configuration->quiet) {
-	      		pam_syslog(pamh, LOG_ERR,
+				pam_syslog(pamh, LOG_ERR,
                        "pam_set_item() failed %s", pam_strerror(pamh, rv));
-				snprintf(password_prompt, sizeof(password_prompt), _("Error 2332: setting PAM userentry failed"));
-				pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+				pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2332: setting PAM userentry failed"));
 				sleep(configuration->err_display_time);
 			}
 	    goto auth_failed_nopw;
@@ -637,8 +626,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
           ERR1("match_user() failed: %s", get_error());
 			if (!configuration->quiet) {
 				pam_syslog(pamh, LOG_ERR, "match_user() failed: %s", get_error());
-				snprintf(password_prompt, sizeof(password_prompt), _("Error 2334: No matching user"));
-				pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+				pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2334: No matching user"));
 				sleep(configuration->err_display_time);
 			}
 	  goto auth_failed_nopw;
@@ -659,8 +647,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 		if (!configuration->quiet) {
 			pam_syslog(pamh, LOG_ERR,
 				"no valid certificate which meets all requirements found");
-		snprintf(password_prompt, sizeof(password_prompt), _("Error 2336: No matching certificate found"));
-		pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+		pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2336: No matching certificate found"));
 		sleep(configuration->err_display_time);
 	}
     goto auth_failed_nopw;
@@ -669,8 +656,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 
   /* if signature check is enforced, generate random data, sign and verify */
   if (configuration->policy.signature_policy) {
-		snprintf(password_prompt, sizeof(password_prompt), _("Checking signature"));
-		pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
+		pam_prompt(pamh, PAM_TEXT_INFO, NULL, _("Checking signature"));
 
 
 #ifdef notdef
@@ -690,10 +676,9 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
       ERR1("get_random_value() failed: %s", get_error());
 		if (!configuration->quiet){
 			pam_syslog(pamh, LOG_ERR, "get_random_value() failed: %s", get_error());
-			snprintf(password_prompt, sizeof(password_prompt), _("Error 2338: Getting random value failed"));
-			pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+			pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2338: Getting random value failed"));
 			sleep(configuration->err_display_time);
-	 	}
+		}
       goto auth_failed_nopw;
     }
 
@@ -705,8 +690,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
       ERR1("sign_value() failed: %s", get_error());
 		if (!configuration->quiet) {
 			pam_syslog(pamh, LOG_ERR, "sign_value() failed: %s", get_error());
-			snprintf(password_prompt, sizeof(password_prompt), _("Error 2340: Signing failed"));
-			pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+			pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2340: Signing failed"));
 			sleep(configuration->err_display_time);
 		}
       goto auth_failed_nopw;
@@ -725,8 +709,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
       ERR1("verify_signature() failed: %s", get_error());
 		if (!configuration->quiet) {
 			pam_syslog(pamh, LOG_ERR, "verify_signature() failed: %s", get_error());
-			snprintf(password_prompt, sizeof(password_prompt), _("Error 2342: Verifying signature failed"));
-			pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+			pam_prompt(pamh, PAM_ERROR_MSG , NULL, _("Error 2342: Verifying signature failed"));
 			sleep(configuration->err_display_time);
 		}
       return PAM_AUTH_ERR;
@@ -807,8 +790,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     ERR1("close_pkcs11_session() failed: %s", get_error());
 		if (!configuration->quiet) {
 			pam_syslog(pamh, LOG_ERR, "close_pkcs11_module() failed: %s", get_error());
-			snprintf(password_prompt, sizeof(password_prompt), _("Error 2344: Closing PKCS#11 session failed"));
-			pam_prompt(pamh, PAM_ERROR_MSG , NULL, password_prompt);
+			pam_prompt(pamh, PAM_ERROR_MSG , NULL, ("Error 2344: Closing PKCS#11 session failed"));
 			sleep(configuration->err_display_time);
 		}
     return pkcs11_pam_fail;
