@@ -17,7 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
 
-/* $Id: card_eventmgr.c 474 2010-11-18 09:17:38Z ludovic.rousseau $ */
+/* $Id: card_eventmgr.c 484 2011-01-22 17:48:57Z ludovic.rousseau $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -210,7 +210,6 @@ static int parse_args(int argc, char *argv[]) {
 
 	/* and now re-parse command line to take precedence over cfgfile */
         for (i = 1; i < argc; i++) {
-	    int res;
             if (strcmp("daemon", argv[i]) == 0) {
 		daemonize=1;
 	  	continue;
@@ -224,11 +223,11 @@ static int parse_args(int argc, char *argv[]) {
 	  	continue;
 	    }
             if (strstr(argv[i],"timeout=") ) {
-                res=sscanf(argv[i],"timeout=%d",&timeout);
+                sscanf(argv[i],"timeout=%d",&timeout);
                 continue;
             }
             if (strstr(argv[i],"timeout_limit=") ) {
-                res=sscanf(argv[i],"timeout_limit=%d",&timeout_limit);
+                sscanf(argv[i],"timeout_limit=%d",&timeout_limit);
                 continue;
             }
 	    if (strstr(argv[i],"pidfile=") ) {
@@ -258,7 +257,6 @@ static pid_t read_pidfile(char *filename)
 {
     FILE *fd;
     pid_t pid;
-    int res;
     long temp = 0;
 
     fd = fopen(filename, "r");
@@ -268,7 +266,7 @@ static pid_t read_pidfile(char *filename)
 	return 0;
     }
 
-    res=fscanf(fd, "%ld", &temp);
+    fscanf(fd, "%ld", &temp);
     pid = temp;
 
     fclose(fd);
@@ -285,7 +283,6 @@ static void remove_pidfile(char *filename)
 static void create_pidfile(char *filename)
 {
     int fd;
-    int res;
     char tmp[20];
 
     fd = open(filename, O_WRONLY | O_CREAT | O_EXCL, 0644);
@@ -298,7 +295,7 @@ static void create_pidfile(char *filename)
     snprintf(tmp, sizeof(tmp)-1, "%d\n", getpid());
     tmp[sizeof(tmp)-1] = '\0';
 
-    res=write(fd, tmp, strlen(tmp));
+    write(fd, tmp, strlen(tmp));
 
     close(fd);
 }
