@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pkcs11_inspect.c 441 2010-08-12 21:45:03Z ludovic.rousseau $
+ * $Id: pkcs11_inspect.c 503 2011-10-18 11:50:10Z ludovic.rousseau $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -103,14 +103,12 @@ int main(int argc, const char **argv) {
     return 1;
   }
 
-#ifdef HAVE_NSS
   /* not really needed, but.... */
   rv = pkcs11_pass_login(ph,configuration->nullok);
   if (rv != 0) {
     ERR1("pkcs11_pass_login() failed: %s", get_error());
     return 2;
   }
-#endif
 
   /* get certificate list (cert space is owned by ph) */
   certs = get_certificate_list(ph, &cert_count);
@@ -129,7 +127,7 @@ int main(int argc, const char **argv) {
   for (i = 0; i < cert_count; i++) {
     X509 *x509 = get_X509_certificate(certs[i]);
     if (x509 != NULL) {
-      DBG1("verifing the certificate #%d", i + 1);
+      DBG1("verifying the certificate #%d", i + 1);
       /* verify certificate (date, signature, CRL, ...) */
       rv = verify_certificate(x509, &configuration->policy);
       if (rv < 0) {
